<?php
defined( 'ABSPATH' ) || exit;

class DozentLMS_Addon_EDD extends DozentLMS_Addon {

	private $edd_post_types;

	public function __construct() {
		$this->id         = 'edd';
		$this->name       = __( 'Easy Digital Downloads', 'dozent-lms' );
		$this->description
		                  = __( 'Sell courses with Easy Digital Downloads (EDD). Easy digital downloads is an eCommerce solution for WordPress. EDD is best known for selling digital items.',
			'dozent-lms' );
		$this->dependency = [
			'easy-digital-downloads/easy-digital-downloads.php' => 'Easy Digital Downloads',
		];

		parent::__construct();
	}

	public function init() {
		$this->edd_post_types = apply_filters( 'edd_download_metabox_post_types', [ 'download' ] );

		add_action( 'add_meta_boxes', [ $this, 'courses_list_edd_metabox' ] );

		foreach ( $this->edd_post_types as $post_type ) {
			add_action( "save_post_$post_type", [ $this, 'save_edd_download' ] );
		}
		add_filter( 'dozentlms_course_price', [ $this, 'course_price' ], 10, 2 );

		add_action( 'dozentlms_closed_course_enroll', [ $this, 'redirect_to_product' ], 10, 2 );
		add_action( 'edd_insert_payment', [ $this, 'insert_enrollment_data' ], 10, 2 );
		add_action( 'edd_update_payment_status', [ $this, 'update_enrollment_status' ], 10, 3 );
		add_action( 'edd_payment_delete', [ $this, 'cancel_enrollment_by_payment' ] );
	}

	public function courses_list_edd_metabox() {
		foreach ( $this->edd_post_types as $post_type ) {
			add_meta_box( 'dlms_add_courses_to_edd', __( 'Dozent LMS Courses', 'dozent-lms' ),
				[ $this, 'dlms_add_courses_to_edd' ], $post_type, 'normal', 'high' );
		}
	}

	public function dlms_add_courses_to_edd() {
		include DOZENTLMS_ABSPATH . '/addons/edd/courses-to-add-metabox.php';
	}

	public function save_edd_download( $download_id ) {
		global $wpdb;

		// Dont' save meta boxes for revisions or autosaves.
		if ( ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) || is_int( wp_is_post_revision( $download_id ) )
		     || is_int( wp_is_post_autosave( $download_id ) ) ) {
			return;
		}

		if ( 'courses_panel' !== dozentlms_request_text( 'dozentlms_edd_panel' ) ) {
			return;
		}


		/**
		 * Attaching courses to the product.
		 */

		$courses = dozentlms_request_arr( 'edd_data_panel_courses' );
		if ( is_array( $courses ) && count( $courses ) ) {
			foreach ( $courses as $course_id ) {
				update_post_meta( $course_id, '_product_id', $download_id );
				update_post_meta( $course_id, '_product_platform', 'EDD' );
			}
		}

		/**
		 * Deleting Courses which are no longer attached with a product.
		 */

		$attached_courses = dozentlms_get_course_ids_by_product( $download_id );

		if ( is_array( $courses )
		     && is_array( $attached_courses )
		     && count( $attached_courses )
		     && count( $attached_courses ) ) {
			$detached_courses = array_diff( $attached_courses, $courses );

			if ( is_array( $detached_courses ) && count( $detached_courses ) ) {

				$in_meta_key = "'" . implode( "','", [ '_product_id', '_product_platform' ] ) . "'";
				$in_post_ids = "'" . implode( "','", $detached_courses ) . "'";

				$wpdb->query( "DELETE FROM $wpdb->postmeta WHERE meta_key IN({$in_meta_key}) AND post_id IN({$in_post_ids}) " );
			}
		}
	}

	/**
	 * Get the price from EDD download
	 *
	 * @param $price
	 * @param $course
	 */
	public function course_price( $price, $course ) {

		if ( dozentlms_is_course_closed( $course ) ) {
			//Get the Course price from the EDD Download Price

			$product = dozentlms_get_course_product( $course );
			if ( 'EDD' === $product->platform && ! empty( $product->product ) ) {
				$price = edd_price( $product->product_id, false );
			}
		}

		return $price;
	}

	public function insert_enrollment_data( $payment_id, $payment_data ) {
		if ( ! is_user_logged_in() ) {
			return false;
		}

		$downloads = dozentlms_arr_get( $payment_data, 'downloads' );
		$user_id   = get_current_user_id();

		foreach ( $downloads as $download ) {
			$download_id = $download['id'];
			$course_ids  = dozentlms_get_course_ids_by_product( $download_id );

			foreach ( $course_ids as $course_id ) {
				dozentlms_enroll_course( $course_id, $user_id,
					[ 'order_id' => $payment_id, 'order_platform' => 'EDD' ] );
			}
		}

	}


	public function update_enrollment_status( $payment_id, $new_status, $old_status ) {
		$enrollments = get_posts( [
			'numberposts' => -1,
			'post_type'   => 'dlms_enrollment',
			'post_status' => array_keys( dozentlms_enrollment_statuses() ),
			'meta_query'  => [
				[
					'key'   => '_belongs_order_id',
					'value' => $payment_id,
				]
			]
		] );

		$enrollment_status = 'dlms-cancelled';

		if ( in_array( $new_status, [ 'pending', 'processing' ] ) ) {
			$enrollment_status = 'dlms-pending';
		} elseif ( in_array( $new_status, [ 'publish', 'complete' ] ) ) {
			$enrollment_status = 'dlms-active';
		}

		if ( ! empty( $enrollments ) ) {
			foreach ( $enrollments as $enrollment ) {
				wp_update_post( [ 'ID' => $enrollment->ID, 'post_status' => $enrollment_status ] );
			}
		}
	}

	/**
	 * Cancel enrollment after delete a payment
	 *
	 * @param $payment_id
	 */

	public function cancel_enrollment_by_payment( $payment_id ){
		$enrollments = get_posts( [
			'numberposts' => -1,
			'post_type'   => 'dlms_enrollment',
			'post_status' => array_keys( dozentlms_enrollment_statuses() ),
			'meta_query'  => [
				[
					'key'   => '_belongs_order_id',
					'value' => $payment_id,
				]
			]
		] );

		$enrollment_status = 'dlms-cancelled';

		if ( ! empty( $enrollments ) ) {
			foreach ( $enrollments as $enrollment ) {
				wp_update_post( [ 'ID' => $enrollment->ID, 'post_status' => $enrollment_status ] );
			}
		}
	}


	public function redirect_to_product( $course_id, $user_id ) {
		if ( dozentlms_has_course_access( $course_id, $user_id ) ) {
			return;
		}

		$product = dozentlms_get_course_product( $course_id );
		if ( 'EDD' !== $product->platform || empty( $product->product ) ) {
			return;
		}

		wp_redirect( get_the_permalink( $product->product_id ) );
		exit();
	}
}

dozentlms_addon_register( 'DozentLMS_Addon_EDD' );