<?php
defined( 'ABSPATH' ) || exit;

/**
 * Get all addons for Dozent LMS registered by `dozentlms_addon_register()`
 *
 * @see dozentlms_addon_register();
 *
 * @since 1.0.0
 *
 *
 * @return DozentLMS_Addon[]
 */

function dozentlms_addons() {
	$addon_classes = apply_filters( 'dozentlms_addons', [] );

	return $addon_classes;
}

/**
 * Returns true if addon is activated
 *
 * @param string $addon_id Add-On ID or Key
 *
 * @since DozentLMS 1.1.0
 *
 * @return bool
 */

function dozentlms_addon_activated( $addon_id ) {
	$activated_addons = (array) get_option( 'dozentlms_active_addons' );
	$activated_addons = array_filter( $activated_addons );

	return in_array( $addon_id, $activated_addons );
}

function dozentlms_addon_register( $class ) {
	$object = new $class;

	add_filter( 'dozentlms_addons', function ( $classes ) use ( $class, $object ) {

		$classes[ $class ] = $object;

		return $classes;
	} );
}


/**
 * Star Rating Add-On
 */

/**
 * @param  float  $rating_value
 * @param  bool  $show_rating_number
 * @param  bool  $echo
 *
 * @return string
 */

function dozentlms_render_rating( $rating_value = 5, $show_rating_number = false, $echo = true ) {

	$html = '<div class="rating-star-wrapper">';

	$html .= '<span class="dlms-sr-only"> ' . sprintf( __( 'Rating: %s out of %s', 'dozent-lms' ), '4.7', '5' )
	         . ' </span>';

	if ( $show_rating_number ) {
		$html .= "<span class='rating-number'> {$rating_value} </span>";
	}

	$rating_range         = range( 1, 5 );
	$should_check_decimal = true;

	$html .= '<span class="star-rating-icons">';
	foreach ( $rating_range as $rating ) {
		if ( $rating_value >= $rating ) {
			$html .= '<i class="lmsicon-star-full" data-rating="' . $rating . '"></i>';
		} else {

			$rating_decimal = $rating_value - intval( $rating_value );
			if ( $rating_decimal >= 0.5 && $should_check_decimal ) {
				$html                 .= '<i class="lmsicon-star-half" data-rating="' . $rating . '"></i>';
				$should_check_decimal = false;
			} else {
				$html .= '<i class="lmsicon-star-empty" data-rating="' . $rating . '"></i>';
			}
		}
	}
	$html .= '</span>';

	$html .= "</div>";

	if ( $echo ) {
		echo $html;
	}

	return $html;
}

/**
 * Render rating input form field
 *
 *
 * @since DozentLMS 1.1.8
 *
 * @param  float  $rating_value
 * @param  bool  $echo
 *
 * @return string
 */

function dozentlms_rating_input( $rating_value = 5, $echo = true ) {

	$field = "<div class='dlms-rating-input-wrap'> " . dozentlms_render_rating( $rating_value, false, false )
	        . " <input type='hidden' name='rating_input_value' value='{$rating_value}'> </div> ";

	if ( $echo ) {
		echo $field;
	}

	return $field;
}

function dozentlms_get_course_rating_by_user( $course = null, $user_id = 0 ) {
	$course    = get_post( $course );
	if ( empty( $course ) ) {
		return null;
	}
	$course_id = $course->ID;

	if ( ! $user_id ) {
		$user_id = get_current_user_id();
	}

	$args = array(
		'status'  => 'all',
		'post_id' => $course_id,
		'user_id' => $user_id,
		'type'    => 'course_review',
		'order'   => 'DESC',
	);

	$query = new WP_Comment_Query;
	$query = $query->query( $args );

	if ( ! empty( $query[0] ) ) {
		$result = $query[0];
		$result->rating = get_comment_meta( $result->comment_ID, '_rating', true );

		return (object) get_object_vars( $query[0] );
	}

	return null;
}

/**
 * Get course rating overview
 *
 * @since DozentLMS 1.1.9
 *
 * @param  null  $course
 *
 * @return array|mixed|string
 */

function dozentlms_get_course_rating_overview( $course = null ) {
	$course = get_post( $course );

	$rating_structure = [
		'total_rating'   => 0,
		'rating_average' => '0.00',
		'rating_stats'   => [
			5 => [
				'total'   => 0,
				'percent' => 0,
			],
			4 => [
				'total'   => 0,
				'percent' => 0,
			],
			3 => [
				'total'   => 0,
				'percent' => 0,
			],
			2 => [
				'total'   => 0,
				'percent' => 0,
			],
			1 => [
				'total'   => 0,
				'percent' => 0,
			],
		]
	];

	if ( empty( $course->ID ) ) {
		return $rating_structure;
	}

	$rating_data = get_post_meta( $course->ID, '_rating_data', true );
	if ( ! empty( $rating_data ) ) {
		return maybe_unserialize( $rating_data );
	}

	return $rating_structure;
}

function dozentlms_course_get_reviews( $args = [] ) {
	global $wpdb;

	$defaults = [
		'user_id'      => 0,
		'course_id'    => 0,
		'search'       => null,
		'rating'       => 'all',
		'status'       => 1,
		'current_page' => 0,
		'start'        => 0,
		'per_page'     => dozentlms_results_per_page(),
		'order_by'     => 'DESC',
	];

	$args = wp_parse_args( $args, $defaults );

	/**
	 * Get start page to query by pagination
	 */

	if ( ! empty( dozentlms_request_text( 'current_page' ) ) ) {
		$args['current_page'] = dozentlms_request_text( 'current_page' );
	}

	$page          = max( 1, $args['current_page'] );
	$args['start'] = ( $page - 1 ) * $args['per_page'];

	extract( $args, EXTR_SKIP );

	$limit_query = '';
	if ( ! empty( $per_page ) && $per_page != '-1' ) {
		$limit_query = " LIMIT {$start}, {$per_page} ";
	}

	$order_by_sql = '';
	if ( ! empty( $order_by ) && in_array( strtoupper( $order_by ), [ 'DESC', 'ASC' ] ) ) {
		$order_by_sql = " ORDER BY rating_tbl.comment_ID {$order_by} ";
	}

	$whereSQL = " WHERE 1 = 1 ";

	if ( ! empty( $course_id ) ) {
		$whereSQL .= " AND rating_tbl.comment_post_ID = {$course_id} ";
	}
	if ( ! empty( $user_id ) ) {
		$whereSQL .= " AND rating_tbl.user_id = {$user_id} ";
	}
	if ( ! empty( $search ) ) {
		$whereSQL .= " AND rating_tbl.comment_content like '%{$search}%'  ";
	}

	$innerJoinSQL = "";
	if ( ! empty( $rating ) && $rating !== 'all' ) {
		$innerJoinSQL = " AND rating_meta_tbl.meta_value = {$rating}  ";
	}

	//TODO, set status value, text to numeric
	if ( ! empty( $status ) && ! in_array( $status, [ 'any', 'all' ] ) ) {
		$whereSQL .= " AND rating_tbl.comment_approved = {$status}  ";
	}

	$total_reviews = $wpdb->get_var( "SELECT COUNT( rating_tbl.comment_ID )
FROM {$wpdb->comments} rating_tbl
INNER JOIN {$wpdb->commentmeta} rating_meta_tbl ON rating_tbl.comment_ID = rating_meta_tbl.comment_id AND rating_meta_tbl.meta_key = '_rating' {$innerJoinSQL}
{$whereSQL} AND rating_tbl.comment_type = 'course_review' " );
	$total_reviews = intval( $total_reviews );

	$reviews = $wpdb->get_results( "SELECT rating_tbl.*, rating_meta_tbl.meta_value as rating, reviewer_tbl.display_name as reviewer_name
FROM {$wpdb->comments} rating_tbl
INNER JOIN {$wpdb->commentmeta} rating_meta_tbl ON rating_tbl.comment_ID = rating_meta_tbl.comment_id AND rating_meta_tbl.meta_key = '_rating' {$innerJoinSQL}
LEFT JOIN {$wpdb->users} reviewer_tbl ON rating_tbl.user_id = reviewer_tbl.ID
{$whereSQL} AND rating_tbl.comment_type = 'course_review' {$order_by_sql} {$limit_query} " );

	$results_end_at = ( $page * $per_page );
	$results_count = is_array( $reviews ) ? count( $reviews ) : 0;

	$results = [
		'page'            => $page,
		'total'           => $total_reviews,
		'count'           => $results_count,
		'results'         => $reviews,
		'results_queried' => $results_end_at,
		'results_left'    => max( ( $total_reviews - $results_end_at ), 0 ),
	];

	return (object) $results;
}