<?php
defined( 'ABSPATH' ) || exit;

class DozentLMS_Addon_Ratings_Reviews extends DozentLMS_Addon {

	public function __construct() {
		$this->id   = 'ratings-reviews';
		$this->name = __( 'Ratings & Reviews', 'dozent-lms' );
		$this->description
		            = __( 'Give the course a star rating, write about the course you have taken and let other learners know what to expect.',
			'dozent-lms' );

		parent::__construct();
	}

	public function init() {
		//Add necessary settings
		add_filter( 'dozentlms_settings_args_general', [ $this, 'add_settings_fields' ] );
		add_action( 'dozentlms/course/header/btn_groups/after', [ $this, 'leave_rating_btn' ] );
		add_action( 'dozentlms/course-page/section/reviews/title/after', [ $this, 'leave_rating_btn' ] );
		add_action( 'dozentlms/learning_area/topbar/after', [ $this, 'leave_rating_btn' ] );
		add_action( 'wp_footer', [ $this, 'inject_review_modal' ] );
		add_action( 'wp_ajax_review_form', [ $this, 'save_review_ajax' ] );
		add_action( 'wp_ajax_dozentlms_show_more_reviews', [ $this, 'show_more_reviews' ] );
		add_action( 'wp_ajax_dozentlms_filter_reviews', [ $this, 'filter_reviews' ] );
		add_action( 'dozentlms/course/card/meta/after', [ $this, 'rating_at_the_card' ] );
		add_action( 'dozentlms/list-course/title/after', [ $this, 'rating_at_the_card' ] );

		//Add given ratings to the dashboard
		add_filter( 'dozentlms_dashboard_menu_items', [ $this, 'my_ratings_to_dashboard' ] );
	}


	public function add_settings_fields( $fields ) {

		$fields['form_fields']['ratings_reviews'] = [

			'title'        => __( 'Ratings &amp; reviews', 'dozent-lms' ),
			'description'  => __( 'Course ratings and reviews settings', 'dozent-lms' ),
			'input_fields' => [

				'who_can_rate_course' => [
					'type'        => 'radio',
					'label'       => __( 'Who can rate the course', 'dozent-lms' ),
					'title'       => __( 'Enable', 'dozent-lms' ),
					'description' => __( 'Determine who can place ratings and write reviews on the course.',
						'dozent-lms' ),
					'options'     => [
						'logged_users'         => __( 'Anyone who logged in', 'dozent-lms' ),
						'enrolled_students'    => __( 'Enrolled Students', 'dozent-lms' ),
						'who_completed_course' => __( 'Students who completed course', 'dozent-lms' ),
					],

					'default' => 'logged_users'
				],

				'show_ratings_on_the_course_card' => [
					'type'        => 'switch',
					'label'       => __( 'Show ratings on the course card', 'dozent-lms' ),
					'description' => __( 'Whether star rating will show on the course card',
						'dozent-lms' ),
				],


			],
		];

		return $fields;

	}

	public function leave_rating_btn() {
		if ( ! is_user_logged_in() ) {
			return;
		}

		dozentlms_load_template( 'addons/ratings-reviews/leave-rating-btn' );
	}

	public function inject_review_modal() {
		if (
			! is_user_logged_in()
			&& ( ! dozentlms_is_course()
			     || ! dozentlms_is_syllabus_singular() )
		) {
			return;
		}

		dozentlms_load_template( 'addons/ratings-reviews/course-page-review-modal' );
	}

	public function save_review_ajax() {
		$course_id = dozentlms_request_text( 'course_id' );
		$rating    = dozentlms_request_text( 'rating_input_value' );
		$review    = dozentlms_request_text( 'course_review' );
		$user_id   = get_current_user_id();

		$data = [
			'comment_post_ID'   => $course_id,
			'comment_content'   => $review,
			'comment_author_IP' => dozentlms_get_client_ip(),
			'comment_approved'  => 1,
			'comment_agent'     => 'DozentLMS',
			'comment_type'      => 'course_review',
			'user_id'           => $user_id,
			'comment_meta'      => [
				'_rating' => $rating,
			]
		];

		$current_review = dozentlms_get_course_rating_by_user( $course_id );

		if ( empty( $current_review ) ) {
			wp_insert_comment( $data );
		} else {
			$data['comment_ID'] = $current_review->comment_ID;
			wp_update_comment( $data );
		}

		$this->_sync_reviews_with_course( $course_id );
		wp_send_json_success( [ 'message' => __( 'The review has been saved', 'dozent-lms' ) ] );
	}

	/**
	 * Sync a course rating and save it to post meta
	 *
	 * @param $course_id
	 */

	public function _sync_reviews_with_course( $course_id ) {
		global $wpdb;

		$initial_rating = $wpdb->get_row( $wpdb->prepare( "SELECT COUNT(rating_tbl.comment_ID) as total_rating, SUM(rating_meta_tbl.meta_value) as rating_sum
FROM {$wpdb->comments} rating_tbl
INNER JOIN {$wpdb->commentmeta} rating_meta_tbl ON rating_tbl.comment_ID = rating_meta_tbl.comment_id AND rating_meta_tbl.meta_key = '_rating'
WHERE rating_tbl.comment_post_ID = %d AND comment_approved = 1 AND rating_tbl.comment_type = 'course_review' ",
			$course_id ) );

		$total_rating = intval( $initial_rating->total_rating );
		$rating_sum   = intval( $initial_rating->rating_sum );

		$average_rating = 0.00;
		if ( $total_rating ) {
			$average_rating = bcdiv( $rating_sum / $total_rating, 1, 2 );
		}

		$count_five_star = (int) $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(rating_tbl.comment_ID) as total_rating
FROM {$wpdb->comments} rating_tbl
INNER JOIN {$wpdb->commentmeta} rating_meta_tbl ON rating_tbl.comment_ID = rating_meta_tbl.comment_id AND rating_meta_tbl.meta_key = '_rating' AND rating_meta_tbl.meta_value = 5
WHERE rating_tbl.comment_post_ID = %d AND comment_approved = 1 AND rating_tbl.comment_type = 'course_review' ",
			$course_id ) );

		$count_four_star = (int) $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(rating_tbl.comment_ID) as total_rating
FROM {$wpdb->comments} rating_tbl
INNER JOIN {$wpdb->commentmeta} rating_meta_tbl ON rating_tbl.comment_ID = rating_meta_tbl.comment_id AND rating_meta_tbl.meta_key = '_rating' AND rating_meta_tbl.meta_value = 4
WHERE rating_tbl.comment_post_ID = %d AND comment_approved = 1 AND rating_tbl.comment_type = 'course_review' ",
			$course_id ) );

		$count_three_star = (int) $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(rating_tbl.comment_ID) as total_rating
FROM {$wpdb->comments} rating_tbl
INNER JOIN {$wpdb->commentmeta} rating_meta_tbl ON rating_tbl.comment_ID = rating_meta_tbl.comment_id AND rating_meta_tbl.meta_key = '_rating' AND rating_meta_tbl.meta_value = 3
WHERE rating_tbl.comment_post_ID = %d AND comment_approved = 1 AND rating_tbl.comment_type = 'course_review' ",
			$course_id ) );

		$count_two_star = (int) $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(rating_tbl.comment_ID) as total_rating
FROM {$wpdb->comments} rating_tbl
INNER JOIN {$wpdb->commentmeta} rating_meta_tbl ON rating_tbl.comment_ID = rating_meta_tbl.comment_id AND rating_meta_tbl.meta_key = '_rating' AND rating_meta_tbl.meta_value = 2
WHERE rating_tbl.comment_post_ID = %d AND comment_approved = 1 AND rating_tbl.comment_type = 'course_review' ",
			$course_id ) );

		$count_one_star = (int) $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(rating_tbl.comment_ID) as total_rating
FROM {$wpdb->comments} rating_tbl
INNER JOIN {$wpdb->commentmeta} rating_meta_tbl ON rating_tbl.comment_ID = rating_meta_tbl.comment_id AND rating_meta_tbl.meta_key = '_rating' AND rating_meta_tbl.meta_value = 1
WHERE rating_tbl.comment_post_ID = %d AND comment_approved = 1 AND rating_tbl.comment_type = 'course_review' ",
			$course_id ) );

		/**
		 * Calculate rating percentage
		 */
		$one_star_percent = 0;
		if ( $count_one_star > 0 ) {
			$one_star_percent = intval( ( $count_one_star * 100 ) / $total_rating );
		}

		$two_star_percent = 0;
		if ( $count_two_star > 0 ) {
			$two_star_percent = intval( ( $count_two_star * 100 ) / $total_rating );
		}

		$three_star_percent = 0;
		if ( $count_three_star > 0 ) {
			$three_star_percent = intval( ( $count_three_star * 100 ) / $total_rating );
		}

		$four_star_percent = 0;
		if ( $count_four_star > 0 ) {
			$four_star_percent = intval( ( $count_four_star * 100 ) / $total_rating );
		}

		$five_star_percent = 0;
		if ( $count_five_star > 0 ) {
			$five_star_percent = intval( ( $count_five_star * 100 ) / $total_rating );
		}

		/**
		 * Prepare the rating data.
		 */
		$rating_structure = [
			'total_rating'   => $total_rating,
			'rating_average' => $average_rating,
			'rating_stats'   => [
				5 => [
					'total'   => $count_five_star,
					'percent' => $five_star_percent,
				],
				4 => [
					'total'   => $count_four_star,
					'percent' => $four_star_percent,
				],
				3 => [
					'total'   => $count_three_star,
					'percent' => $three_star_percent,
				],
				2 => [
					'total'   => $count_two_star,
					'percent' => $two_star_percent,
				],
				1 => [
					'total'   => $count_one_star,
					'percent' => $one_star_percent,
				],
			]
		];

		update_post_meta( $course_id, '_rating_data', $rating_structure );
		update_post_meta( $course_id, '_rating_total', $total_rating );
		update_post_meta( $course_id, '_rating_average', $average_rating );
	}

	public function show_more_reviews() {
		$course_id = dozentlms_request_text( 'course_id' );
		$page      = max( 1, dozentlms_request_text( 'page' ) ) + 1;

		$args = [
			'course_id'    => $course_id,
			'current_page' => $page,
		];

		$reviews = dozentlms_course_get_reviews( $args );


		$HTML = '';

		if ( is_array( $reviews->results ) && count( $reviews->results ) ) {
			foreach ( $reviews->results as $review ) {
				$HTML .= dozentlms_loop_review( $review, null, false );
			}
		}

		$reviews->results_html = $HTML;

		wp_send_json_success( $reviews );
	}

	public function filter_reviews() {
		$course_id = dozentlms_request_text( 'course_id' );
		$search    = dozentlms_request_text( 'search_text' );
		$rating    = dozentlms_request_text( 'rating' );

		$reviews = dozentlms_course_get_reviews( compact( 'course_id', 'search', 'rating' ) );

		$HTML = '';

		if ( is_array( $reviews->results ) && count( $reviews->results ) ) {
			foreach ( $reviews->results as $review ) {
				$HTML .= dozentlms_loop_review( $review, null, false );
			}
		} else {
			$HTML .= '<div class="dlms-no-reviews-found-wrap">';
			$HTML .= '<h3> ' . __( 'No reviews were found at this search criteria', 'dozent-lms' ) . ' </h3>';
			$HTML .= '</div>';
		}

		$reviews->results_html = $HTML;

		wp_send_json_success( $reviews );
	}

	public function rating_at_the_card() {
		$show = (bool) dozentlms_get_setting( 'show_ratings_on_the_course_card' );
		if ( ! $show ) {
			return;
		}

		$course_rating = dozentlms_get_course_rating_overview( get_the_ID() );

		$total_rating   = dozentlms_arr_get( $course_rating, 'total_rating' );
		$rating_average = dozentlms_arr_get( $course_rating, 'rating_average' );

		$html = "<div class='course-card-rating-row'>";

		$html .= dozentlms_render_rating( $rating_average, true, false );
		$html .= "<p class='card-rating-total'> ({$total_rating}) </p>";

		$html .= "</div>";

		echo $html;
	}

	public function my_ratings_to_dashboard( $menu_items ) {
		$menu_items['user']['menu_items']['my-ratings'] = [ 'menu_title' => __( 'My Ratings', 'dozent-lms' ), 'icon' => 'lmsicon-star-half' ];

		return $menu_items;
	}

}

dozentlms_addon_register( 'DozentLMS_Addon_Ratings_Reviews' );