<?php
defined( 'ABSPATH' ) || exit;

class DozentLMS_Addon_WooCommerce extends DozentLMS_Addon {

	public function __construct() {
		$this->id          = 'woocommerce';
		$this->name        = __( 'WooCommerce - v.3.0', 'dozent-lms' );
		$this->description = __( 'WooCommerce is an open-source e-commerce plugin for WordPress. This Add-On allows you to sell your courses through the WooCommerce.', 'dozent-lms' );
		$this->dependency  = [
			'woocommerce/woocommerce.php' => 'WooCommerce',
		];

		parent::__construct();
	}

	public function init() {
		add_filter( 'woocommerce_product_data_tabs', [ $this, 'data_tabs_courses' ] );
		add_action( 'woocommerce_product_data_panels', [ $this, 'data_panel_courses' ] );
		add_action( 'save_post_product', [ $this, 'save_product' ] );

		add_filter( 'dozentlms_course_price', [ $this, 'course_price' ], 10, 2 );
		add_action( 'dozentlms_closed_course_enroll', [ $this, 'redirect_to_product' ], 10, 2 );

		add_action( 'woocommerce_checkout_update_order_meta', [ $this, 'enroll_after_purchase' ] );
		add_action( 'woocommerce_order_status_changed', [ $this, 'order_status_track' ], 10, 3 );
	}

	public function data_tabs_courses( $tabs ) {

		$tabs['dozentlms_courses'] = array(
			'label'    => __( 'Dozent LMS Courses', 'dozent-lms' ),
			'target'   => 'dlms-course-panel',
			'priority' => 10,
		);

		return $tabs;
	}

	public function data_panel_courses() {
		include DOZENTLMS_ABSPATH . '/addons/woocommerce/courses-data-panel.php';
	}

	public function save_product( $postID ) {
		global $wpdb;

		// Dont' save meta boxes for revisions or autosaves.
		if ( ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) || is_int( wp_is_post_revision( $postID ) )
		     || is_int( wp_is_post_autosave( $postID ) ) ) {
			return;
		}

		if ( 'courses_panel' !== dozentlms_request_text( 'dozentlms_wc_data_panel' ) ) {
			return;
		}

		/**
		 * Attaching courses to the product.
		 */

		$courses = dozentlms_request_arr( 'wc_data_panel_courses' );
		if ( is_array( $courses ) && count( $courses ) ) {
			foreach ( $courses as $course_id ) {
				update_post_meta( $course_id, '_product_id', $postID );
				update_post_meta( $course_id, '_product_platform', 'WooCommerce' );
			}
		}

		/**
		 * Deleting Courses which are no longer attached with a product.
		 */

		$attached_courses = dozentlms_get_course_ids_by_product( $postID );

		if ( is_array( $courses )
		     && is_array( $attached_courses )
		     && count( $attached_courses )
		     && count( $attached_courses ) ) {
			$detached_courses = array_diff( $attached_courses, $courses );

			if ( is_array( $detached_courses ) && count( $detached_courses ) ) {

				$in_meta_key = "'" . implode( "','", [ '_product_id', '_product_platform' ] ) . "'";
				$in_post_ids = "'" . implode( "','", $detached_courses ) . "'";

				$wpdb->query( "DELETE FROM $wpdb->postmeta WHERE meta_key IN({$in_meta_key}) AND post_id IN({$in_post_ids}) " );
			}
		}

	}

	/**
	 * @param  string  $price  Price
	 * @param  WP_Post  $course  Course Object
	 *
	 * @return string
	 */

	public function course_price( $price, $course ) {
		if ( dozentlms_is_course_closed( $course ) ) {
			//Get the Course price from the WooCommerce...

			$product = dozentlms_get_course_product( $course );
			if ( 'WooCommerce' === $product->platform && ! empty( $product->product ) ) {
				$price = $product->product->get_price_html();
			}
		}

		return $price;
	}

	public function redirect_to_product( $course_id, $user_id ) {
		if ( dozentlms_has_course_access( $course_id, $user_id ) ) {
			return;
		}

		$product = dozentlms_get_course_product( $course_id );
		if ( 'WooCommerce' !== $product->platform || empty( $product->product ) ) {
			return;
		}

		wp_redirect( get_the_permalink( $product->product_id ) );
		exit();
	}

	public function enroll_after_purchase( $order_id ) {
		if ( ! is_user_logged_in() ) {
			return false;
		}

		$order   = wc_get_order( $order_id );
		$user_id = get_current_user_id();

		$ordered_items = $order->get_items();
		foreach ( $ordered_items as $product ) {
			$product_id = $product['product_id'];
			$course_ids = dozentlms_get_course_ids_by_product( $product_id );

			foreach ( $course_ids as $course_id ) {
				dozentlms_enroll_course( $course_id, $user_id, [ 'order_id' => $order_id ] );
			}

		}
	}

	/**
	 * WooCommerce has updated the order status, so update the enrollment accordingly
	 */
	public function order_status_track( $order_id, $old_status, $new_status ) {
		$enrollments = get_posts( [
			'numberposts' => -1,
			'post_type'   => 'dlms_enrollment',
			'post_status' => array_keys( dozentlms_enrollment_statuses() ),
			'meta_query'  => [
				[
					'key'   => '_belongs_order_id',
					'value' => $order_id,
				]
			]
		] );

		$enrollment_status = 'dlms-cancelled';

		if ( in_array( $new_status, [ 'pending', 'processing', 'on-hold' ] ) ) {
			$enrollment_status = 'dlms-pending';
		} elseif ( 'completed' === $new_status ) {
			$enrollment_status = 'dlms-active';
		}

		if ( ! empty( $enrollments ) ) {
			foreach ( $enrollments as $enrollment ) {
				wp_update_post( [ 'ID' => $enrollment->ID, 'post_status' => $enrollment_status ] );
			}
		}

	}

}

dozentlms_addon_register( 'DozentLMS_Addon_WooCommerce' );