<?php

namespace DozentLMS\Core\Course;

class Builder {

	public static function instance() {
		static $instance = null;

		if ( null === $instance ) {
			$instance = new self();
		}

		return $instance;
	}

	public function __construct() {
		add_action( 'wp_ajax_dozentlms_add_module', [ $this, 'add_module' ] );
		add_action( 'wp_ajax_dozentlms_update_module', [ $this, 'update_module' ] );
		add_action( 'wp_ajax_dozentlms_delete_module', [ $this, 'delete_module' ] );
		add_action( 'wp_ajax_dozentlms_get_wp_editor', [ $this, 'get_wp_editor' ] );
		add_action( 'wp_ajax_dozentlms_lecture_save', [ $this, 'save_lecture' ] );
		add_action( 'wp_ajax_sorting_course_contents', [ $this, 'sort_contents' ] );
		add_action( 'wp_ajax_dozentlms_delete_module_item', [ $this, 'delete_item' ] );

		add_action( 'wp_ajax_dozentlms_get_module_item_form', [ $this, 'item_form' ] );
	}

	public function add_module() {
		if ( empty( dozentlms_request_text( 'module_name' ) ) ) {
			wp_send_json_error();
		}

		$module_name        = dozentlms_request_text( 'module_name' );
		$module_description = dozentlms_request_textarea( 'module_description' );
		$course_id          = (int) dozentlms_request_text( 'course_id' );
		$module_order_id    = dozentlms_next_module_order( $course_id );

		$post_arr  = apply_filters( 'dozentlms_new_module_attr', [
			'post_type'    => 'dozentlms_module',
			'post_title'   => $module_name,
			'post_content' => $module_description,
			'post_status'  => 'publish',
			'post_author'  => get_current_user_id(),
			'post_parent'  => $course_id,
			'menu_order'   => $module_order_id,
		] );
		$module_id = wp_insert_post( $post_arr );

		ob_start();
		require DOZENTLMS_ABSPATH . '/core/course/contents.php';
		$course_contents = ob_get_clean();

		wp_send_json_success( [ 'contents' => $course_contents ] );
	}

	public function update_module() {
		if ( empty( dozentlms_request_text( 'module_name' ) ) ) {
			wp_send_json_error();
		}

		$module_id          = dozentlms_request_text( 'module_id' );
		$module_name        = dozentlms_request_text( 'module_name' );
		$module_description = dozentlms_request_textarea( 'module_description' );

		$postarr = apply_filters( 'dozentlms_update_module_attr', [
			'ID'           => $module_id,
			'post_title'   => $module_name,
			'post_content' => $module_description,
		] );
		wp_update_post( $postarr );

		ob_start();
		$course_id          = dozentlms_request_text( 'course_id' );
		require DOZENTLMS_ABSPATH . '/core/course/contents.php';
		$course_contents = ob_get_clean();

		wp_send_json_success( [ 'contents' => $course_contents ] );
	}

	public function delete_module() {
		$module_id = (int) dozentlms_request_text( 'module_id' );
		wp_delete_post( $module_id, true );
		wp_send_json_success();
	}

	public function get_wp_editor() {
		ob_start();
		wp_editor( '', 'lecture_content', [ 'textarea_rows' => 5 ] );
		wp_send_json_success( [ 'wp_editor' => ob_get_clean() ] );
	}

	public function save_lecture() {
		$course_id       = dozentlms_request_text( 'course_id' );
		$module_id       = dozentlms_request_text( 'module_id' );
		$lecture_id      = dozentlms_request_text( 'lecture_id' );
		$title           = dozentlms_request_text( 'title' );
		$lecture_content = wp_kses_post( $_POST['lecture_content'] );
		$_video          = dozentlms_request_arr( '_video' );

		if ( empty( trim( $title ) ) ) {
			wp_send_json_error( [ 'message' => __( 'Lecture title required', 'dozent-lms' ) ] );
		}

		/**
		 * Video duration info
		 */

		$video_source = dozentlms_arr_get( $_video, 'source' );
		if ( $video_source === 'upload' ) {
			$video_id   = dozentlms_arr_get( $_video, 'upload.video_id' );
			$video_info = wp_prepare_attachment_for_js( $video_id );
			if ( ! empty( $video_info ) ) {
				$duration = dozentlms_arr_get( $video_info, 'fileLength' );

				if ( ! empty( $duration ) && is_string( $duration ) ) {

					$hour   = '';
					$minute = '';
					$second = '';

					$duration_parts = explode( ':', $duration );
					$duration_count = count( $duration_parts );

					if ( 3 === $duration_count ) {
						list( $hour, $minute, $second ) = $duration_parts;
					} elseif ( 2 === $duration_count ) {
						list( $minute, $second ) = $duration_parts;
					}

					$media_runtime = [
						'hh' => $hour,
						'mm' => $minute,
						'ss' => $second,
					];

					$_video['runtime'] = $media_runtime;
				}
			}
		}

		/**
		 * Lecture Arguments
		 */

		$post_arr = [
			'post_type'    => 'dlms_lecture',
			'post_title'   => $title,
			'post_content' => $lecture_content,
			'post_status'  => 'publish',
			'post_author'  => get_current_user_id(),
			'post_parent'  => $module_id,
			'meta_input'   => [
				'_video' => $_video,
			],
		];

		if ( $lecture_id ) {
			$lecture                = get_post( $lecture_id );
			$post_arr['ID']         = $lecture_id;
			$post_arr['menu_order'] = $lecture->menu_order;
			unset( $post_arr['post_author'] );
		} else {
			$menu_order_id          = dozentlms_next_content_order_id( $course_id );
			$post_arr['menu_order'] = $menu_order_id;
		}

		$post_name = sanitize_title( $title );
		$post_arr[ 'post_name' ] = $post_name;

		$lecture_id = wp_insert_post( $post_arr );

		//Sync the course again
		dozentlms_sync_course( $course_id );

		ob_start();
		include DOZENTLMS_ABSPATH . '/core/course/contents.php';
		wp_send_json_success( [ 'lecture_id' => $lecture_id, 'contents' => ob_get_clean() ] );
	}

	public function delete_item(){
		$item_id = dozentlms_request_text( 'item_id' );
		wp_delete_post( $item_id, true );
		wp_send_json_success();
	}

	public function item_form() {
		$item_id = dozentlms_request_text( 'item_id' );

		$item_type = get_post_type( $item_id );

		ob_start();

		/**
		 * Module Item Edit form path, like lecture|quiz|assignments
		 *
		 * @since 1.0.0
		 *
		 * @param  int  $item_id  Item ID
		 * @param  string  $item_type  Item Post Type
		 */

		include apply_filters( 'dozentlms_module_item_edit_form_path',
			DOZENTLMS_ABSPATH . '/core/course/edit-lecture.php', $item_id, $item_type );
		$formHtml = ob_get_clean();

		wp_send_json_success( [ 'html' => $formHtml ] );
	}

	public function sort_contents() {
		global $wpdb;

		$modules = dozentlms_request_arr( 'modules' );

		if ( dozentlms_arr_count( $modules ) ) {
			$item_i = 1;

			foreach ( $modules as $key => $module ) {
				$module_id = dozentlms_arr_get( $module, 'module_id' );
				$wpdb->update( $wpdb->posts, [ 'menu_order' => $key ], [ 'ID' => $module_id ] );
				$item_ids = dozentlms_arr_get( $module, 'item_ids' );
				if ( dozentlms_arr_count( $item_ids ) ) {
					foreach ( $item_ids as $item_id ) {
						$wpdb->update( $wpdb->posts, [ 'post_parent' => $module_id, 'menu_order' => $item_i ],
							[ 'ID' => $item_id ] );
						$item_i ++;
					}
				}
			}
		}
	}

}