<?php

/**
 * Access array and get the value by array key
 *
 * @since DozentLMS 1.0.0
 *
 * @param array $array   An array from which we want to retrieve value.
 * @param int|string $key Array key which we would like to return
 * @param  null  $default Default value Null
 *
 * @return array|mixed|null
 */

function dozentlms_arr_get( $array, $key, $default = null ) {
	$array = (array) $array;

	if ( ! is_array( $array ) ) {
		return $default;
	}

	if ( is_null( $key ) ) {
		return $array;
	}

	if ( array_key_exists( $key, $array ) ) {
		return $array[ $key ];
	}

	if ( strpos( $key, '.' ) === false ) {
		return isset( $array[ $key ] ) ? $array[ $key ] : $default;
	}

	$segments = explode( '.', $key );

	foreach ( $segments as $segment ) {
		if ( is_array( $array ) && array_key_exists( $segment, $array ) ) {
			$array = $array[ $segment ];
		} else {
			return $default;
		}
	}

	return $array;
}

function dozentlms_arr_count( $array = [] ) {
	if ( empty( $array ) || ! is_array( $array ) ) {
		return 0;
	}

	return count( $array );
}

function dozentlms_request_arr( $field ) {
	if ( isset( $_POST[ $field ] ) ) {
		return dozentlms_arr_sanitize( $_POST[ $field ] );
	}

	return null;
}

function dozentlms_arr_sanitize( $array ) {
	if ( dozentlms_arr_count( $array ) ) {
		foreach ( $array as $key => $value ) {

			if ( is_string( $value ) ) {
				if ( $value != strip_tags( $value ) ) {
					//Contains HTML, sanitize with wp_kses_post
					$arr[ $key ] = wp_kses_post( $value );
				} else {
					if ( strpos( $value, "\n" ) !== false ) {
						//Found Textarea Text with line break
						//Sanitize with sanitize_textarea_field();
						$arr[ $key ] = sanitize_textarea_field( $value );

					} else {
						//Just sanitize with sanitize_text_field();
						$arr[ $key ] = sanitize_text_field( $value );
					}
				}

			} elseif ( is_array( $value ) ) {
				$arr[ $key ] = dozentlms_arr_sanitize( $value );
			}
		}
	}

	return $array;
}

function dozentlms_get_the_post_ID( $post = null ) {

	if ( $post instanceof WP_Post ) {
		return $post->ID;
	} elseif ( is_object( $post ) && ! empty( $post->ID ) ) {
		return $post->ID;
	}

	if ( ! empty( $post ) && is_numeric( $post ) ) {
		return $post;
	}

	if ( ! empty( get_the_ID() ) ) {
		return get_the_ID();
	}

	return false;
}

function dozentlms_get_video_sources() {
	$videos = [
		'upload'   => __( 'Upload Video', 'dozent-lms' ),
		'youtube'  => __( 'YouTube', 'dozent-lms' ),
		'vimeo'    => __( 'Vimeo', 'dozent-lms' ),
		'embedded' => __( 'Embedded Code', 'dozent-lms' ),
	];

	return apply_filters( 'dozentlms_video_sources', $videos );
}

function dozentlms_get_enabled_video_sources() {
	$videos = dozentlms_get_video_sources();

	return apply_filters( 'dozentlms_get_enabled_video_sources', $videos );
}

function dozentlms_placeholder_src() {
	return apply_filters( 'dozentlms_placeholder_src', DOZENTLMS_URL . 'resources/images/placeholder.jpeg' );
}


/**
 * Retrieve the template path based on priority template.
 *
 * Searches in the STYLESHEETPATH before TEMPLATEPATH and wp-includes/plugins/dozent-lms/{$template}
 * so that themes which inherit from a parent theme can just overload one file.
 *
 * @since 1.0.0
 *
 * @param string $template_name Template file(s) to search for, in order.
 * @param bool         $load           If true the template file will be loaded if it is found.
 *
 * @param array        $args           Optional. Additional arguments passed to the template.
 *                                     Default empty array.
 * @param bool $echo Weather loaded template should echo or return the generated HTML.
 *
 * @return string The template filename if one is located.
 */

function dozentlms_locate_template( $template_name = null, $load = false, $args = [], $echo = true ) {
	$file_name = str_replace( '.php', '', $template_name ) . '.php';

	if ( file_exists( trailingslashit( STYLESHEETPATH ) . 'dozent-lms/' . $file_name ) ) {
		$located = trailingslashit( STYLESHEETPATH ) . 'dozent-lms/' . $file_name;
	} elseif ( file_exists( trailingslashit( TEMPLATEPATH ) . 'dozent-lms/' . $file_name ) ) {
		$located = trailingslashit( TEMPLATEPATH ) . 'dozent-lms/' . $file_name;
	} else {
		$located = DOZENTLMS_ABSPATH . "/templates/{$file_name}";
	}

	if ( $load && '' !== $located ) {

		if ( is_array( $args ) && ! empty( $args ) ) {
			extract( $args );
		}

		if ( $echo ) {
			require $located;
		} else {
			ob_start();
			require $located;

			return apply_filters( 'dozentlms_template_load', ob_get_clean(), $located, $template_name );
		}
	}

	if ( $template_name ) {
		return apply_filters( 'dozentlms_template_path', $located, $template_name );
	}

	return '';
}

function dozentlms_load_template( $template_name = null, $args = [], $echo = true ) {
	return dozentlms_locate_template( $template_name, true, $args, $echo );
}

function dozentlms_get_current_page_number() {
	$paged = get_query_var( 'paged' );

	$current_page_number = 1;
	if ( $paged ) {
		$current_page_number = (int) $paged;
	}

	return $current_page_number;
}

/**
 * Get the number of results per page
 *
 * @since DozentLMS 1.1.8
 *
 * @return mixed|void
 */
function dozentlms_results_per_page() {
	$per_page = dozentlms_get_setting( 'pagination_results_per_page', 20 );

	return apply_filters( 'dozentlms_results_per_page', $per_page );
}

/**
 * Returns whether a particular user or current user has the administrator capability.
 *
 * This function also accepts an ID of an object to check against if the capability
 *
 * Example usage:
 *
 *     dozentlms_is_administrator( $user->ID );
 *     dozentlms_is_administrator( $user );
 *
 * @since 1.2.0
 *
 * @param int|WP_User $user       User ID or object.
 * @return bool Whether the user has the administrator capability.
 */

function dozentlms_is_administrator( $user = null ) {
	if ( $user ) {
		return user_can( $user, 'administrator' );
	}

	return current_user_can( 'administrator' );
}


/**
 * Returns whether a particular user or current user has the instructor capability.
 *
 * This function also accepts an ID of an object to check against if the capability
 *
 * Example usage:
 *
 *     dozentlms_is_instructor( $user->ID );
 *     dozentlms_is_instructor( $user );
 *
 * @since 1.0.0
 *
 * @param int|WP_User $user       User ID or object.
 * @return bool Whether the user has the instructor capability.
 */

function dozentlms_is_instructor( $user = null ) {
	if ( $user ) {
		return user_can( $user, 'dozentlms_instructor' );
	}

	return current_user_can( 'dozentlms_instructor' );
}

function dozentlms_is_course_instructor( $course = null, $user = null ) {

	$user = new WP_User( $user );
	if ( empty( $user->ID ) && is_user_logged_in() ) {
		$user = wp_get_current_user();
	}

	$instructor_ids = dozentlms_course_instructor_ids( $course );

	return in_array( $user->ID, $instructor_ids );
}

/**
 * TODO: get instructor IDs from the actually instructors list assigned for a course
 *
 * @param  null  $course
 *
 * @return array
 */

function dozentlms_course_instructor_ids( $course = null ) {
	$course = get_post( $course );

	return (array) $course->post_author;
}

function dozentlms_set_redirect_to( $redirect_to = null ){
	if ( null === $redirect_to ) {
		setcookie( 'dlms_redirect_to', $redirect_to, 1 ); //Set redirect for one hour
	} else {
		setcookie( 'dlms_redirect_to', $redirect_to, time() + 1800, '/' ); //Set redirect for half hour
	}
}

function dozetnlms_get_redirect_to(){
	return ( ! empty( $_COOKIE[ 'dlms_redirect_to' ] ) ) ? sanitize_text_field( $_COOKIE[ 'dlms_redirect_to' ] ) : null;
}


/**
 * Get client IP address
 *
 * @since DozentLMS 1.1.8
 *
 * @return string
 */

function dozentlms_get_client_ip() {
	$ipaddress = '127.0.0.1';

	if ( isset( $_SERVER['HTTP_X_REAL_IP'] ) ) {
		$ipaddress = sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_REAL_IP'] ) );
	} elseif ( isset( $_SERVER['HTTP_CLIENT_IP'] ) ) {
		$ipaddress = sanitize_text_field( wp_unslash( $_SERVER['HTTP_CLIENT_IP'] ) );
	} elseif ( isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
		$ipaddress = sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_FORWARDED_FOR'] ) );
	} elseif ( isset( $_SERVER['HTTP_X_FORWARDED'] ) ) {
		$ipaddress = sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_FORWARDED'] ) );
	} elseif ( isset( $_SERVER['HTTP_FORWARDED_FOR'] ) ) {
		$ipaddress = sanitize_text_field( wp_unslash( $_SERVER['HTTP_FORWARDED_FOR'] ) );
	} elseif ( isset( $_SERVER['HTTP_FORWARDED'] ) ) {
		$ipaddress = sanitize_text_field( wp_unslash( $_SERVER['HTTP_FORWARDED'] ) );
	} elseif ( isset( $_SERVER['REMOTE_ADDR'] ) ) {
		$ipaddress = sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) );
	}

	return apply_filters( 'dozentlms_get_client_ip', $ipaddress );
}

/**
 * Format the text before print it
 * Un-quotes a quoted string
 * Generate <br /> tags for new line
 *
 * @since DozentLMS 1.2.0
 *
 * @param  string  $text Original Text
 * @param  bool  $nl2br should text convert new line to br tag
 *
 * @return mixed|void
 */

function dozentlms_text_output( $text = '', $nl2br = true ) {
	if ( ! $nl2br ) {
		return stripslashes( $text );
	}

	return nl2br( stripslashes( $text ) );
}

/**
 * Get the Dashboard current page slug
 *
 * @return mixed|string|null
 */

function dozentlms_get_current_dashboard_page_slug() {
	if ( ! dozentlms_is_frontend_dashboard() ) {
		return null;
	}

	$slug  = get_query_var( 'dbsegment' );
	$slug2 = get_query_var( 'dbsegment2' );

	if ( empty( $slug2 ) ) {
		return $slug;
	}

	return trailingslashit( $slug ) . $slug2;
}

/**
 * Returns true when the page is loaded via ajax.
 *
 * @return bool
 */

function dozentlms_is_ajax() {
	return function_exists( 'wp_doing_ajax' ) ? wp_doing_ajax() : ( defined( 'DOING_AJAX' ) && DOING_AJAX );
}

/**
 * Retrieve all of the DozentLMS supported post statuses.
 *
 *
 * @since 1.2.2
 *
 * @return string[] Array of post status labels keyed by their status.
 */

function dozentlms_get_post_statuses() {
	$statuses                 = get_post_statuses();
	$statuses['draft_course'] = __( 'Draft Course', 'dozent-lms' );

	return $statuses;
}