<?php
/**
 * Users actions and activities class
 */

namespace DozentLMS\Core\User;

class Activities {

	public static function instance() {
		static $instance = null;

		if ( null === $instance ) {
			$instance = new self();
		}

		return $instance;
	}

	function __construct() {
		add_action( 'wp_ajax_dozentlms_toggle_wishlist', [ $this, 'toggle_wishlist' ] );

		if ( ! is_user_logged_in() ) {
			add_action( 'wp_footer', [ $this, 'login_modal' ] );
		}

		//Dashboard Profile Edit
		$this->edit_profile();

		//Admin Activities
		add_action( 'admin_init', [ $this, 'instructor_personalization' ] );

		//Instructor List Activity
		add_action( 'admin_init', [ $this, 'instructor_activity' ] );

		//Change Password From Dashboard
		add_action( 'template_redirect', [ $this, 'change_password' ] );
	}

	public function toggle_wishlist() {
		$user_id   = get_current_user_id();
		$course_id = dozentlms_request_text( 'course_id' );

		$wishlisted = get_user_meta( $user_id, '_dozentlms_course_wishlist' );

		if ( in_array( $course_id, $wishlisted ) ) {
			//Remove from wishlist
			delete_user_meta( $user_id, '_dozentlms_course_wishlist', $course_id );

			$btn_text = __( 'Wishlist', 'dozent-lms' );
			$btn_text .= " <i class='lmsicon-heart-line'></i>";
		} else {
			add_user_meta( $user_id, '_dozentlms_course_wishlist', $course_id );

			$btn_text = __( 'Wishlisted', 'dozent-lms' );
			$btn_text .= " <i class='lmsicon-heart'></i>";
		}

		wp_send_json_success( [ 'btn_text' => $btn_text ] );
	}

	public function login_modal() {
		dozentlms_load_template( 'includes/login-modal' );
	}

	public function edit_profile() {
		$action = dozentlms_request_text( 'dlms_action' );
		if ( 'edit_profile' !== $action || ! is_user_logged_in() ) {
			return;
		}

		$user_id = get_current_user_id();

		$first_name  = dozentlms_request_text( 'first_name' );
		$last_name   = dozentlms_request_text( 'last_name' );
		$description = dozentlms_request_textarea( 'description' );

		update_user_meta( $user_id, 'first_name', $first_name );
		update_user_meta( $user_id, 'last_name', $last_name );
		update_user_meta( $user_id, 'description', $description );

		wp_redirect( wp_get_referer() );
		exit();
	}

	public function instructor_personalization() {
		if ( dozentlms_is_instructor() && ! current_user_can( 'administrator' ) ) {
			add_filter( 'posts_clauses', [ $this, 'instructors_posts_clauses' ] );
		}
	}

	public function instructors_posts_clauses( $clauses ) {
		global $wpdb;

		$where_clause = isset( $clauses['where'] ) ? $clauses['where'] : '';

		$user_id            = get_current_user_id();
		$own_courses_clause = " AND {$wpdb->posts}.post_author = $user_id ";

		$clauses['where'] = $where_clause . $own_courses_clause;

		return $clauses;
	}

	public function instructor_activity() {
		$page    = dozentlms_get_var( 'page' );
		$user_id = dozentlms_get_var( 'user_id' );
		$action  = dozentlms_get_var( 'action' );

		if ( 'dozentlms-instructors' !== $page || empty( $user_id ) || empty( $action ) ) {
			return;
		}

		if ( 'do_approve' === $action ) {

			dozentlms_approve_instructor( $user_id );

		} elseif ( 'do_reject' === $action ) {
			update_user_meta( $user_id, '_instructor_role', 'rejected' );

			$instructor = new \WP_User( $user_id );
			$instructor->remove_role( 'dozentlms_instructor' );
		}

		wp_redirect( admin_url( 'admin.php?page=dozentlms-instructors' ) );
		die();
	}

	public function change_password() {
		$action = dozentlms_request_text( 'dlms_action' );
		if ( 'change_password' !== $action || ! is_user_logged_in() ) {
			return;
		}

		$current_password     = dozentlms_request_text( 'current_password' );
		$new_password         = dozentlms_request_text( 'new_password' );
		$confirm_new_password = dozentlms_request_text( 'confirm_new_password' );

		$errors = [];

		if ( empty( $current_password ) ) {
			$errors['current_password'] = __( 'Current Password field is required', 'dozent-lms' );
		}
		if ( empty( $new_password ) ) {
			$errors['new_password'] = __( 'New Password field is required', 'dozent-lms' );
		} elseif ( strlen( $new_password ) < 6 ) {
			$errors['new_password'] = __( 'New Password must be at least 6 characters', 'dozent-lms' );
		}

		if ( empty( $confirm_new_password ) ) {
			$errors['confirm_new_password'] = __( 'Confirm New Password field is required', 'dozent-lms' );
		}
		if ( ! empty( $new_password ) && ! empty( $confirm_new_password )
		     && ( $new_password != $confirm_new_password ) ) {
			$errors['confirm_new_password'] = __( 'New Password does not match with Confirm New Password',
				'dozent-lms' );
		}

		if ( is_array( $errors ) && count( $errors ) ) {
			add_filter( 'dozentlms_change_password_errors', function () use ( $errors ) {
				return $errors;
			} );

			return;
		}

		/**
		 * Getting the current user and checking against new password.
		 */

		$user = wp_get_current_user();
		if ( wp_check_password( $current_password, $user->user_pass ) ) {
			wp_set_password( $new_password, $user->ID );
			dozentlms_set_flash( __( 'Your password has been changed successfully', 'dozent-lms' ) );

			//Set login cache again
			wp_signon( [ 'user_login' => $user->user_login, 'user_password' => $new_password, 'remember' => true ],
				is_ssl() );
		} else {
			dozentlms_set_flash( __( 'Your current password invalid', 'dozent-lms' ) );
		}

		wp_redirect( dozentlms_dashboard_link( 'manage-profile/change-password/' ) );
		exit();
	}

}