<?php
/**
 * User Dashboard Panel Area
 */

namespace DozentLMS\Core\User;

class Panel {

	public static function menu_slugs() {
		$plucked_items = wp_list_pluck( self::menu_items(), 'menu_items' );

		return array_keys( call_user_func_array( 'array_merge', $plucked_items ) );
	}

	public static function raw_menu_items() {
		$raw_items = [];

		foreach ( self::menu_items() as $menu_group ) {
			$menu_items = dozentlms_arr_get( $menu_group, 'menu_items' );
			$capability = dozentlms_arr_get( $menu_group, 'capability' );

			foreach ( $menu_items as $menu_slug => $menu_item ) {
				if ( is_array( $menu_item ) ) {
					$menu_item['capability'] = $capability;
				} elseif ( is_string( $menu_item ) ) {
					$menu_item = [
						'menu_title' => $menu_item,
						'capability' => $capability,
					];
				}

				$raw_items[ $menu_slug ] = $menu_item;
			}
		}

		return $raw_items;
	}

	public static function render_sidebar_menu() {
		$menu_groups = self::menu_items();
		$dashboard_link = dozentlms_dashboard_link();
		$dbsegment  = get_query_var( 'dbsegment' );
		$active_class = empty( $dbsegment ) ? 'active' : '';

		$html = "<ul class='dlms-dashboard-menu'>";

		$html .= "<li class='dlms-dashboard-menu-item menu-item-dashboard {$active_class} '> <a href='{$dashboard_link}'> <i class='lmsicon-dashboard'></i> <span class='dashboard-menu-item-title'> " . __( 'Dashboard', 'dozent-lms' ) . " </span> </a> </li>";

		foreach ( $menu_groups as $menu_group_key => $menu_group ) {
			$menu_items = dozentlms_arr_get( $menu_group, 'menu_items' );
			$capability = dozentlms_arr_get( $menu_group, 'capability' );

			if ( $capability && ! current_user_can( $capability ) ) {
				continue;
			}

			foreach ( $menu_items as $menu_slug => $menu_item ) {
				$hide_in_ui = is_array( $menu_item ) ? dozentlms_arr_get( $menu_item, 'hide_in_ui' ) : false;

				if ( $hide_in_ui ) {
					continue;
				}

				$active_class = $dbsegment === $menu_slug ? 'active' : '';
				$menu_title = is_array( $menu_item ) ? dozentlms_arr_get( $menu_item, 'menu_title' ) : $menu_item;

				if ( is_array( $menu_item ) && ! empty( $menu_item['link'] ) ) {
					$menu_link = $menu_item['link'];
				} else {
					$menu_link = dozentlms_dashboard_link( $menu_slug );
				}

				$icon = dozentlms_arr_get( $menu_item, 'icon' );
				$icon_html = "<i class='lmsicon-circle-right'></i>";
				if ( ! empty( $icon ) ) {
					$icon_html = "<i class='{$icon}'></i>";
				}

				$html .= "<li class='dlms-dashboard-menu-item menu-item-{$menu_slug} {$active_class} ' > <a href='{$menu_link}'> {$icon_html} <span class='dashboard-menu-item-title'> {$menu_title} </span> </a> </li>";
			}
		}


		$html .= "</ul>";

		return $html;
	}

	public static function menu_items() {
		$items = [
			'instructors' => [
				'priority'         => 1,
				'menu_group_title' => __( 'Instructor Area', 'dozent-lms' ),
				'capability'       => 'dozentlms_instructor',
				'menu_items'       => [
					'create-new-course'        => [
						'menu_title' => __( 'New Course', 'dozent-lms' ),
						'icon'  => 'lmsicon-course-add',
					],
					'edit-course'              => [
						'menu_title' => __( 'Edit Courses', 'dozent-lms' ),
						'hide_in_ui' => true,
					],
					'my-courses-as-instructor' => [ 'menu_title' => __( 'My Courses', 'dozent-lms' ), 'icon' => 'lmsicon-graduation-cap' ],
				],

			],

			'user' => [
				'priority'         => 2,
				'menu_group_title' => __( 'User Area', 'dozent-lms' ),
				'menu_items'       => [
					'my-enrolled-courses' => [ 'menu_title' => __( 'My Enrolled Courses', 'dozent-lms' ), 'icon' => 'lmsicon-list' ],
					'wishlisted-courses'  => [ 'menu_title' => __( 'Wishlist', 'dozent-lms' ), 'icon' => 'lmsicon-heart' ],
					'completed-courses'   => [ 'menu_title' => __( 'Completed Courses', 'dozent-lms' ), 'icon'  => 'lmsicon-success' ],
					'manage-profile'      => [ 'menu_title' => __( 'Manage Profile', 'dozent-lms' ), 'icon' => 'lmsicon-user-tie' ],
				],
			],

			'actions' => [

				'priority'         => 3,
				'menu_group_title' => __( 'Logout', 'dozent-lms' ),
				'menu_items'       => [
					'logout' => [
						'menu_title' => __( 'Logout', 'dozent-lms' ),
						'link'       => wp_logout_url( dozentlms_dashboard_link() ),
						'icon'  => 'lmsicon-exit'
					],

				],

			],

		];

		/**
		 * Filter dashboard menu items
		 * Add or remove menu items by using this filter
		 *
		 * @since DozentLMS 1.0.0
		 *
		 * @param  array  $items
		 */

		$items = apply_filters( 'dozentlms_dashboard_menu_items', $items );

		$sorted_items = [];
		foreach ( $items as $key => $item ) {
			$sorted_items[] = [ $key => $item ];
		}

		usort( $sorted_items, [ __CLASS__, 'build_sorter' ] );

		$items = [];
		foreach ( $sorted_items as $sort_item ) {
			$items[ key( $sort_item ) ] = reset( $sort_item );
		}

		return $items;
	}


	private static function build_sorter( $a, $b ) {
		$a = intval( dozentlms_arr_get( reset( $a ), 'priority' ) );
		$b = intval( dozentlms_arr_get( reset( $b ), 'priority' ) );

		if ( $a == $b ) {
			return 0;
		} elseif ( $a > $b ) {
			return 1;
		} else {
			return - 1;
		}
	}


}