<?php
defined( 'ABSPATH' ) || exit;

function dozentlms_enrollment_statuses( $enrollment_status = null ) {

	$enrollment_statuses = apply_filters( 'dozentlms_enrollment_statuses', [
		'dlms-active'    => [
			'label'                     => __( 'Active Enrollment', 'dozent-lms' ),
			'public'                    => false,
			'show_in_admin_all_list'    => false,
			'show_in_admin_status_list' => false,
			'exclude_from_search'       => true,
		],
		'dlms-pending'   => [
			'label'                     => __( 'Pending Enrollment', 'dozent-lms' ),
			'public'                    => false,
			'show_in_admin_all_list'    => false,
			'show_in_admin_status_list' => false,
			'exclude_from_search'       => true,
		],
		'dlms-expired'   => [
			'label'                     => __( 'Expired Enrollment', 'dozent-lms' ),
			'public'                    => false,
			'show_in_admin_all_list'    => false,
			'show_in_admin_status_list' => false,
			'exclude_from_search'       => true,
		],
		'dlms-cancelled' => [
			'label'                     => __( 'Cancelled Enrollment', 'dozent-lms' ),
			'public'                    => false,
			'show_in_admin_all_list'    => false,
			'show_in_admin_status_list' => false,
			'exclude_from_search'       => true,
		],
	] );

	if ( ! empty( $enrollment_status ) ) {
		if ( ! empty( $enrollment_statuses[ $enrollment_status ] ) ) {
			return $enrollment_statuses[ $enrollment_status ];
		}

		return null;
	}

	return $enrollment_statuses;
}

/**
 * Get course ids attached to a product.
 *
 *
 * @since DozentLMS 1.0.0
 *
 * @param int $product_id Product ID
 *
 * @return array
 */

function dozentlms_get_course_ids_by_product( $product_id = 0 ) {
	if ( ! $product_id ) {
		$product_id = get_the_ID();
	}

	global $wpdb;

	return $wpdb->get_col( $wpdb->prepare( "SELECT post_id as course_id from $wpdb->postmeta WHERE meta_value = %d ", $product_id ) );
}

function dozentlms_get_courses_has_no_product( $skip_product_id = null ) {
	global $wpdb;

	$skip_sql = "";
	if ( ! empty( $skip_product_id ) ) {
		$skip_sql = " AND meta_value != {$skip_product_id} ";
	}

	$exclude_lists = $wpdb->get_col( "SELECT post_id 
FROM $wpdb->postmeta 
WHERE  meta_key = '_product_id' {$skip_sql} " );

	$args = [
		'post_type'      => 'dozentlms_course',
		'post_status'    => 'publish',
		'post__not_in'   => wp_parse_id_list( $exclude_lists ),
		'posts_per_page' => - 1,

		'meta_query' => array(
			array(
				'key'   => '_course_access_type',
				'value' => 'closed',
			),
		),
	];

	$get_posts = new WP_Query;

	return $get_posts->query( $args );
}

class DozentLMS_Course_Product{

	/**
	 * Product Object
	 *
	 * @var WP_Post|WC_Product
	 */

	public $product;

	/**
	 * Attached Product ID
	 *
	 * @var int
	 */

	public $product_id;

	/**
	 * Product Platform
	 *
	 * @var string
	 */
	public $platform;
}

/**
 * Get the product attached to a course
 *
 * @since DoznetLMS 1.0.0
 *
 * @param  WP_Post|int|null  $course Course ID or Course object alias of WP Post Object
 *
 * @return DozentLMS_Course_Product|bool
 */

function dozentlms_get_course_product( $course = null ) {
	$course = get_post( $course );
	$result = new DozentLMS_Course_Product();

	if ( ! empty( $course->ID ) ) {
		$product_id = get_post_meta( $course->ID, '_product_id', true );
		$platform   = get_post_meta( $course->ID, '_product_platform', true );

		if ( ! empty( $product_id ) ) {
			$result->product_id = intval( $product_id );
			$result->platform   = $platform;

			$product = null;

			if ( 'WooCommerce' === $platform && function_exists( 'wc_get_product' ) ) {
				$product = wc_get_product( $product_id );
			} elseif ( 'EDD' === $platform && function_exists( 'edd_get_download' ) ) {
				$product = edd_get_download( $product_id );
			}

			$result->product = apply_filters( 'dozentlms_course_product', $product, $course, $product_id, $platform );
		}
	}

	return apply_filters( 'dozentlms_course_product_object', $result, $course );
}

/**
 * Check if course archive has been filtered
 *
 * @since DozentLMS 1.1.3
 *
 * @return bool
 */

function dozentlms_archive_is_filtered() {
	$requested_vars
		= array_keys( dozentlms_arr_sanitize( $_REQUEST ) ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput

	$get_vars = [ 'per_page', 'sort_by' ];

	if ( count( array_diff( $get_vars, $requested_vars ) ) < count( $get_vars ) ) {
		return true;
	}

	return false;
}


/**
 * Convert a duration to human readable format.
 * Input HH:mm:ss to time integer
 *
 * @since DozentLMS 1.1.5
 *
 * @param string $duration Duration will be in string format (HH:ii:ss) OR (ii:ss),
 *                         with a possible prepended negative sign (-).
 * @return int total duration in time
 */
function dozentlms_duration_to_time( $duration = '' ) {
	//Modified from human_readable_duration();

	if ( ( empty( $duration ) || ! is_string( $duration ) ) ) {
		return false;
	}

	$duration = trim( $duration );

	// Remove prepended negative sign.
	if ( '-' === substr( $duration, 0, 1 ) ) {
		$duration = substr( $duration, 1 );
	}

	if ( ':' === substr( $duration, 0, 1 ) ) {
		$duration = substr( $duration, 1 );
	}

	// Extract duration parts.
	$duration_parts = array_filter( array_reverse( explode( ':', $duration ) ) );
	$duration_count = count( $duration_parts );

	$hour   = null;
	$minute = null;
	$second = null;

	if ( 3 === $duration_count ) {
		// Validate HH:ii:ss duration format.
		if ( ! ( (bool) preg_match( '/^([0-9]+):([0-5]?[0-9]):([0-5]?[0-9])$/', $duration ) ) ) {
			return false;
		}
		// Three parts: hours, minutes & seconds.
		list( $second, $minute, $hour ) = $duration_parts;
	} elseif ( 2 === $duration_count ) {
		// Validate ii:ss duration format.
		if ( ! ( (bool) preg_match( '/^([0-5]?[0-9]):([0-5]?[0-9])$/', $duration ) ) ) {
			return false;
		}
		// Two parts: minutes & seconds.
		list( $second, $minute ) = $duration_parts;
	} elseif ( 1 === $duration_count ) {
		//One Part: Seconds
		list( $second ) = $duration_parts;
	} else {
		return false;
	}

	$duration_in_time = 0;

	if ( is_numeric( $hour ) ) {
		$duration_in_time = ( $hour * 3600 );
	}

	// Add the minute as second
	if ( is_numeric( $minute ) ) {
		$duration_in_time = $duration_in_time + ( $minute * 60 );
	}

	// Add the second part to the string.
	if ( is_numeric( $second ) ) {
		$duration_in_time = $duration_in_time + $second;
	}

	return $duration_in_time;
}

/**
 * Convert time in seconds to duration format
 *
 * show 60 seconds to 00:01:00
 *
 * @since  1.1.5
 *
 * @param  int  $seconds
 *
 * @return string
 */

function dozentlms_time_to_duration( $seconds = 0 ) {
	if ( ! $seconds ) {
		return "00:00";
	}

	$time = round( $seconds );

	if ( intval( ( $time / 3600 ) ) ) {
		return sprintf( '%02d:%02d:%02d', ( $time / 3600 ), ( $time / 60 % 60 ), $time % 60 );
	}

	return sprintf( '%02d:%02d', ( $time / 60 % 60 ), $time % 60 );
}

/**
 * Convert a duration or time in seconds to human readable format.
 * This function is extended version of the default function human_readable_duration();
 *
 * @param string|int $duration Duration will be in string format (HH:ii:ss) OR (ii:ss),
 *                         or time format, time in seconds.
 *
 * @since DozentLMS 1.2.1
 *
 * @see dozentlms_time_to_duration();
 *
 * @param  bool  $show_hour Should show the hours in readable format
 * @param  bool  $show_minute Should show the minutes in readable format
 * @param  bool  $show_seconds Should show the seconds in readable format
 *
 * @return false|string
 */

function dozentlms_human_readable_duration( $duration = '', $show_hour = true, $show_minute = true, $show_seconds = true ) {

	if ( empty( $duration ) ) {
		return false;
	}

	if ( is_integer( $duration ) ) {
		$duration = dozentlms_time_to_duration( $duration );
	}

	$duration = trim( $duration );


	// Remove prepended negative sign.
	if ( '-' === substr( $duration, 0, 1 ) ) {
		$duration = substr( $duration, 1 );
	}

	// Extract duration parts.
	$duration_parts = array_reverse( explode( ':', $duration ) );
	$duration_count = count( $duration_parts );

	$hour   = null;
	$minute = null;
	$second = null;

	if ( 3 === $duration_count ) {
		// Validate HH:ii:ss duration format.
		if ( ! ( (bool) preg_match( '/^([0-9]+):([0-5]?[0-9]):([0-5]?[0-9])$/', $duration ) ) ) {
			return false;
		}
		// Three parts: hours, minutes & seconds.
		list( $second, $minute, $hour ) = $duration_parts;
	} elseif ( 2 === $duration_count ) {
		// Validate ii:ss duration format.
		if ( ! ( (bool) preg_match( '/^([0-5]?[0-9]):([0-5]?[0-9])$/', $duration ) ) ) {
			return false;
		}
		// Two parts: minutes & seconds.
		list( $second, $minute ) = $duration_parts;
	} else {
		return false;
	}

	$human_readable_duration = [];

	// Add the hour part to the string.
	if ( $show_hour && is_numeric( $hour ) ) {
		/* translators: %s: Time duration in hour or hours. */
		$human_readable_duration[] = sprintf( _n( '%s hour', '%s hours', $hour, 'dozent-lms' ), (int) $hour );
	}

	// Add the minute part to the string.
	if ( $show_minute && is_numeric( $minute ) ) {
		/* translators: %s: Time duration in minute or minutes. */
		$human_readable_duration[] = sprintf( _n( '%s minute', '%s minutes', $minute, 'dozent-lms' ), (int) $minute );
	}

	// Add the second part to the string.
	if ( $show_seconds && is_numeric( $second ) ) {
		/* translators: %s: Time duration in second or seconds. */
		$human_readable_duration[] = sprintf( _n( '%s second', '%s seconds', $second, 'dozent-lms' ), (int) $second );
	}

	return implode( ', ', $human_readable_duration );
}

function dozentlms_set_flash( $message = '', $type = 'info' ) {
	if ( headers_sent() ) {
		return;
	}

	$timestamp = current_time( 'timestamp' );
	setcookie( 'dozentlms_session_flash[message]', $message, $timestamp + 300 );
	setcookie( 'dozentlms_session_flash[type]', $type, $timestamp + 300 );
}

function dozentlms_get_flash( $which = 'message' ) {
	if ( empty( $_COOKIE['dozentlms_session_flash'] ) ) { // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized

		return;
	}

	$flash
		= dozentlms_arr_sanitize( $_COOKIE['dozentlms_session_flash'] ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput

	if ( ! is_array( $flash ) || ! count( $flash ) ) {
		return '';
	}

	if ( 'type' === $which ) {
		return dozentlms_arr_get( $flash, 'type' );
	}

	return stripslashes( dozentlms_arr_get( $flash, 'message' ) );
}

function dozentlms_flash_notice() {
	$message = dozentlms_get_flash( 'message' );
	if ( empty( $message ) ) {
		return '';
	}

	$type    = dozentlms_get_flash( 'type' );

	dozentlms_notice( $message, $type );
}